/********************************************************/
/* Program         : iColor.c 							*/
/* Function        : Serial interface to Social Jewlery	*/
/* Author          : Chris Coulston     				*/
/* Platform        : HiTech (Lite Mode)  V9.71a			*/
/* Target          : PIC16LF1825 						*/
/*    			   : 14-pin SMT version					*/
/********************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <htc.h>
#include <pic16lf1825.h>
#include "iColor.h"

/********************************************************/
/* 				Configuration Bits						*/
/********************************************************/
__CONFIG(FOSC_INTOSC & CLKOUTEN_OFF & WDTE_OFF & MCLRE_ON & BOREN_OFF);		// Enable the internal oscillator keep GPIO, turn off WDT
__CONFIG(LVP_OFF & PLLEN_OFF);												// Next invocation of CONFIG modifies second config word


/********************************************************/
/* 				Global Variables						*/
/********************************************************/
unsigned char new_photo = 0;
unsigned char new_photo_set = FALSE;
unsigned char photo_array[PhotoArrayLength];
unsigned char string[48], array[SampleLength];
unsigned char pre = 3;

/********************************************************/
/*					MAIN								*/
/********************************************************/
main() {

	char			letter;
	unsigned char 	i, j, avg, old_avg, first, low, high, flag;
	unsigned char 	next_red, next_blue, next_green, runlength0, runlength1;
	unsigned int	bigsum, delta;

	INIT_PIC();
	low = high = 10;
	letter = 0x30;

	// ---------- Splash screen ----------------
	sprintf(string,"\r\n");
	for (i=0; i< 30; i++) Myprintf(string);
	sprintf(string, "Social Jewlery Interactive Terminal App\r\n");								Myprintf(string);

	while(1) {

		i=RCREG;
		i=RCREG;		// Clear the Receive flag

		while(!RCIF);	/* set when register is not empty */

		switch(RCREG) {

			/***************************************************************************/
			case '?':
				sprintf(string,"\r\n"); 														Myprintf(string);
				sprintf(string,"\r\n------------------------------\r\n");		 				Myprintf(string);
				sprintf(string,"R=%3x	G=%3x	B=%3x	DAC=%x\r\n",CCPR3L,CCPR4L, CCPR2L,DACCON1 & 0x1F); 	Myprintf(string);
				sprintf(string,"?: help menu\r\n");												Myprintf(string);
				sprintf(string,"r/R: decrease/increase red duty cycle\r\n");					Myprintf(string);
				sprintf(string,"g/G: decrease/increase green duty cycle\r\n");					Myprintf(string);
				sprintf(string,"b/B: decrease/increase blue duty cycle\r\n");					Myprintf(string);
				sprintf(string,"s: stream photosensor\r\n");									Myprintf(string);
				sprintf(string,"d/D: decrease/increase DAC output\r\n");						Myprintf(string);
				sprintf(string,"c: turn off all the colors\r\n");								Myprintf(string);
				sprintf(string,"C: cycle through color list\r\n");								Myprintf(string);
				sprintf(string,"t: continuous detection\r\n");									Myprintf(string);
				sprintf(string,"T: last %d collection of reads\r\n", PhotoArrayLength);			Myprintf(string);
				sprintf(string,"a/A: receive/send digital character\r\n");						Myprintf(string);
				sprintf(string,"z: Digital receive bits every %d TMR cnts\r\n",0xFF-SampleInterval);		Myprintf(string);
				sprintf(string,"Z: Free digital listen (O'scope Vphoto)\r\n");					Myprintf(string);
				sprintf(string,"m/M: receive/send a message\r\n");								Myprintf(string);
				sprintf(string,"p/P: decrease/increase light port speed from %d\r\n",pre);		Myprintf(string);
				sprintf(string,"j/J: Jewelry lower RX, upper TX\r\n");							Myprintf(string);
				break;



			/****************************************************************************/
			/* This is that we will want to do to multiplex photodetection with the 	*/
			/* the generation of color sequences.  Works pretty darn good.				*/
			case 'J':
				JewleryTX();
				break;

			case 'j':
				JewleryRX();
				break;

			/***************************************************************************/
			case 'r':
				if (CCPR3L >= 0x10) CCPR3L -= 0x10;		// decrease duty cycle by 6%
				break;
			case 'R':
				if (CCPR3L < MAX_DUTY) CCPR3L += 0x10;	// increase duty cycle by 6%
				break;

			/***************************************************************************/
			case 'g':
				if (CCPR4L >= 0x10) CCPR4L -= 0x10;		// decrease duty cycle by 6%
				break;
			case 'G':
				if (CCPR4L < MAX_DUTY) CCPR4L += 0x10;	// increase duty cycle by 6%
				break;

			/***************************************************************************/
			case 'b':
				if (CCPR2L >= 0x10) CCPR2L -= 0x10;		// decrease duty cycle by 6%
				break;
			case 'B':
				if (CCPR2L < MAX_DUTY) CCPR2L += 0x10;	// increase duty cycle by 6%
				break;

			/***************************************************************************/
			case 'p':
				if (pre > 0) pre -= 1;					// decrease TMR0 prescalar in TX/RX
				break;
			case 'P':
				if (pre < 7) pre += 1;					// increase TMR0 prescalar in TX/RX
				break;


			/***************************************************************************/
			case 's':
				sprintf(string,"Hit any key to stop\r\n");					Myprintf(string);
				
				// Turn off the LEDs
				CCP3CON = 0x00;		RA2 = 0;		// Turn PWM off and pin low
				CCP4CON = 0x00;		RC1 = 0;		// Turn PWM off and pin low
				CCP2CON = 0x00;		RC3 = 0;		// Turn PWM off and pin low
				RA5 = 1;							// Turn on sensing circuit

				while(!RCIF) {
					while(!new_photo);
					new_photo = 0;
					sprintf(string,"%3x:",ADRESH);							Myprintf(string);
					for(i=0; i<ADRESH>>4; i++) putch('*');					sprintf(string,"\r\n");									Myprintf(string);
				}

				RA5 = 0;							// Turn off sensing circuit
				CCP3CON = 0x0C;						// Turn PWM back on
				CCP4CON = 0x0C;						// Turn PWM back on
				CCP2CON = 0x0C;						// Turn PWM back on

				i = RCREG;		// read the break character to clear buffer (and flag)
				break;
				
			/***************************************************************************/
			case 'd':
				DACCON1 = (DACCON1 - 0x01) & 0x1F;
				break;

			/***************************************************************************/
			case 'D':
				DACCON1 = (DACCON1 + 0x01) & 0x1F;
				break;

			/***************************************************************************/
			case 'C':
				ColorCycle();
				break;

			/***************************************************************************/
			case 'c':
				CCPR3L = 0;
				CCPR4L = 0;
				CCPR2L = 0;
				break;

			/****************************************************************************/
			/* This is that we will want to do to multiplex photodetection with the 	*/
			/* the generation of color sequences.  Works pretty darn good.				*/
			case 't':

				sprintf(string,"Read: Hit any key to stop\r\n");					Myprintf(string);
				sprintf(string,"Avg	Old-New	Dev\r\n");									Myprintf(string);

				CCPR3L = 0x80;		// A nice color to pass the time
				CCPR4L = 0X80;
				CCPR2L = 0X80;

				while(!RCIF) {
					while(new_photo_set == FALSE);

					// Now perform average and deviation on the photo data obtained
					new_photo_set = FALSE;
					bigsum = 0;
					for (i=0; i< PhotoArrayLength; i++) bigsum += photo_array[i];
					avg = bigsum >> LogPhotoArrayLength;
					delta = 0;
					for (i=0; i< PhotoArrayLength; i++) {
						if (avg > photo_array[i])	delta += (avg - photo_array[i]);
						else						delta += (photo_array[i] - avg);
					}

					if (avg > old_avg)	j = avg - old_avg;				// How far off
					else				j = old_avg - avg;

					sprintf(string,"%d	%d	%d\r\n",avg,j,delta);					Myprintf(string);
					old_avg = avg;

				} // end detection phase

				i = RCREG;
 
				CCPR3L = 0;
				CCPR4L = 0;
				CCPR2L = 0;

				break;

			/***************************************************************************/
			case 'T':

				bigsum = 0;
				for (i=0; i< PhotoArrayLength; i++) bigsum += photo_array[i];
				avg = bigsum >> LogPhotoArrayLength;
				delta = 0;
				for (i=0; i< PhotoArrayLength; i++) {
					if (avg > photo_array[i])	delta += (avg - photo_array[i]);
					else						delta += (photo_array[i] - avg);
				}

				sprintf(string,"%3d %5d : ",avg,delta);							Myprintf(string);

				for (i=0; i< PhotoArrayLength; i++) {
						sprintf(string,"%x, ",photo_array[i]);					Myprintf(string);
				}
				sprintf(string,"\r\n");											Myprintf(string);
				break;
					

			/***************** Testing for Digital Receive ************************/
			case 'z':
				RunSilent();
				DigitalRXArray(array);
				RunActive();
				for (i=0; i<SampleLength; i++) {
					sprintf(string,"%2x",array[i]);							Myprintf(string); 
					if ( (i & 0x03) == 0x03)  {
						sprintf(string," ");								Myprintf(string); 
					}					
				}
				sprintf(string,"\r\n");										Myprintf(string);
				break;


			/****************************************************************************/
			/* Silence all subsystems and allow V_photo to be examined.					*/
			case 'Z':
				sprintf(string,"Read: Hit any key to stop\r\n");					Myprintf(string);
				RunSilent();

				while(!RCIF);

				i = RCREG;
				RunActive();
			
				break;

			/****************************************************************************/
			/* Simple one character receive for testing									*/
			case 'a':
				RunSilent();
				first = DigitalRX(TRUE);
				RunActive();
				sprintf(string,"bit	low	high\r\n");											Myprintf(string);
				for (i=0; i<10; i++) {
					sprintf(string,"%d	%d	%d\r\n",i,array[i<<1],array[(i<<1)+1]);			Myprintf(string);
				} // end for 

				sprintf(string,"Received: %x	%c\r\n",first,first);						Myprintf(string);
				break;

			/****************************************************************************/
			case 'A':
				RunSilent();
				TX(letter++);
				RunActive();
				break;

			/****************************************************************************/
			/* Multicharacter receive, full implementation								*/
			case 'm':
				sprintf(string,"Waiting for string...\r\n");								Myprintf(string);				
				i = 0;
				flag = FALSE;
				RunSilent();
				// sprintf(string,"i=%x flag = %x string[%x]=%c\r\n",i,flag,i,string[i]);		Myprintf(string);
				while ((i < 32) && (flag == FALSE)) {
					// putch(i+0x30);
					string[i] = DigitalRX(TRUE);
					if (i == 0) {
						TMR1H = 0x00;
						TMR1L = 0x00;
						TMR1IF = 0;
					} // end if first characer
					if (string[i] == 0) break;
					if (string[i] == '\n') break;
					i++;
					if (TMR1IF) flag = TRUE;
				}
				RunActive();
				// sprintf(string,"i=%x flag = %x string[%x]=%c\r\n",i,flag,i,string[i]);		Myprintf(string);

				// if transmission went fine, then print out a message
				if ((string[i] == '\n') && (flag == FALSE)) {
					string[i+1] = 0x00;
					Myprintf(string);
				} else  if (flag == TRUE) {
					sprintf(string,"Transmission time-out.\r\n");							Myprintf(string);
				} else if (string[i] == 0) {
						sprintf(string,"Bit problem.\r\n");									Myprintf(string);
				} else {
						sprintf(string,"Improssbru! string[%x]=%x\r\n",i,string[i]);									Myprintf(string);
				} // end if received time-out

				break;

			/****************************************************************************/
			/* Multicharacter transmit, this is the final implementation				*/
			case 'M':
				sprintf(string,"Enter string.  Hit <Enter> when done.\r\n");				Myprintf(string);
				i = 0;
				while(i < 32) {
					while(!RCIF);
					string[i] = RCREG;
					if (string[i] == '\r') break;
					putch(string[i]);					
					i++;
				}
				string[i] = '\r';
				string[i+1] = '\n';

				RunSilent();
				for (j=0; j<i+2; j++) TX(string[j]);
				RunActive();

				putch('\r');
				putch('\n');
				break;


			/****************************************************************************/
			default:
				sprintf(string,"Huh??\r\n");												Myprintf(string);
				break;

		} // end switch
	} // end while 1
} // end main



/********************************************************/
/*					INIT_PIC							*/
/* Lots of good things going on here, in order they are	*/
/* 1) Configure the internal oscillator to run at 1Mhz	*/
/* 2) Configure the ADC to convert AN3 - V_photo		*/
/* 3) Setup TMR0 to generate an approx 1mS rollover		*/
/* 4) Setup the UART to run at 9600 baud				*/
/* 5) Configure CCP2 to use RC3 as an output for PWM	*/
/* 6) configure the 3 PWM channels on RC1, RC3 and RA2	*/
/* 7) Setup the DAC on RA0								*/
/* 8) Setup the sense pin to switch modes				*/
/* 9) Setup TMR4 to generate a 10mS interrupt			*/
/*10) Setup TMR1 for communication time-out				*/
/*														*/
/*				RC5		CCP1							*/
/*		Blue	RC3		CCP2							*/
/*		Red		RA2		CCP3							*/
/*		Green	RC1		CCP4							*/
/*		v_photo	RA4		output from photo sensor AN3	*/
/*		Sense	RA5		enables red LED to sense		*/
/*				RA0		DAC voltage bias for TIA		*/
/********************************************************/
void INIT_PIC(void) {

	OSCCON = 0b01011000;	// 1Mhz internal clock

	// ---------------------Setup ADC------------------
	/* Setup the ADC */
	TRISA4 = 1;			// ========= Step 1
	ANSA4 = 1;

	ADCS2 = 0;			// ========= Step 2
	ADCS1 = 0;			
	ADCS0 = 0;			// conversion clock = Fosc/2

	ADNREF = 0;			// Voltage reference Vdd
	ADPREF1 = 0;		// and Vss
	ADPREF0 = 0;

	CHS4 = 0;			// ========== Step 3
	CHS3 = 0;			// Select AN3 which is v_photo
	CHS2 = 0;
	CHS1 = 1;
	CHS0 = 1;

	ADON = 1;			// Turn on the ADC

	// ---------Setup TMR0 for 1.024mS rollover------------------
	TMR0CS = 0;			// Clock source Fosc
	PSA = 1;			// no prescalar assigned to TMR0
	PS2 = 0;	PS1 = 1;	PS0 = 1;	// 16:1 prescalar when prescalar is enabled

	// ---------Setup the UART------------------
	RX_PIN = 1;
	TX_PIN = 1;
	SPBRG = DIVIDER;
	RCSTA = 0x90;
	TXSTA = (SPEED|0x20);
	BAUDCON = 0x08;

	// ---------------- setup IO config ----------------
	APFCON0 = 0x10;		// Steer CCP2 towards RC3

	// ---------------- setup 3 PWM channels ----------------
	// Following the prescription on page 219 of the 16F1825 manual
	TRISC3 = 1;			// Set CCP2 to an inputs
	TRISA2 = 1;			// Set CCP3 to an inputs
	TRISC1 = 1;			// Set CCP4 to an inputs

	PR2 = 0xFF;			// Set the PWM period to (256*4)uS

	CCP2CON = 0x0C;		// Enhanced CCP module => PWM enabled
	PSTR2CON = 0x01;

	CCP3CON = 0x0C;		// PWM enabled
	CCP4CON = 0x0C;		// PWM enabled
	
	CCPR2L = 0;					// start at half the rated maximum
	CCPR3L = 0;					// duty cycle to verify all color
	CCPR4L = 0;					// channels are operational

	CCPTMRS	= 0x00;		// Configure all CCP modules to use TMR2
	TMR2 = 0xF0;		// Create a small delay until TMR2 until rollover
	TMR2IF = 0;			// Clear interrupt flag
	T2CON = 0b00000100;	// Turn on TMR2 with no prescalar

	while(!TMR2IF);

	TRISC3 = 0;			// Set CCP2 to an output
	TRISA2 = 0;			// Set CCP3 to an output
	TRISC1 = 0;			// Set CCP4 to an output

	// Setup the DAC according to page 163
	DACCON0 = 0b10100000;
	DACCON1 = 0x1E;
	TRISA0 = 0;

	// Set the sense pin to be an output
	TRISA5 = 0;			// Set the sense pin to be an output
	RA5 = 0;			// and turn sense capability off


	// Setup a 10mS interrupt on rollover
	TMR4 = 0xD8;				// Create a small delay until TMR2 until rollover
	TMR4IF = 0;					// Clear interrupt flag
	TMR4IE = 1;					// Enable interrupts
	T4CON = 0b00000111;			// Turn on TMR4 with 1:64 prescalar
	PR4 = 0xFF;					// Match register for TMR4IF setting

	PEIE = 1;		// Enable perhiphal interrupts
	GIE = 1;		// Enable global interrupts, off to the races

	// Setup Timer1 to detect communication time-out
	TMR1ON = 1;					// Turn on timer 1
	T1CKPS1 = 1;	T1CKPS0=1;	// Timer 1 has a 1:8 prescalar
	TMR1IF = 0;					// Clear the flag to start next interrupts
	
} // end INIT_PIC


/********************************************************/
// This is the ISR for the processor.  Periodic
// timer roll over and acquire another photo voltage
// The delay between shutting off the PWM and enabling the
// sensing circuit seems to require a fair amout of
// tuning.	Const		Delay
//			0xCE		200uS		problems with max green
//			0x9c		400uS		works well
//			0xB5		300uS		Still some weird problems
/********************************************************/
void interrupt MyISR(void) {

	static unsigned char i=0;

	if (TMR4IE) {

		// -----------------------Make a single ADC----------------------------
		// Stop the PWM before the end of the current cycle
		// When the LEDs most likely will be off anyway.  Since
		// it takes 68uS for a ADC to be complete, stop the timer
		// 68uS before rolling over.
		while(TMR2 < 0xD0);
		// while ((RA2 == 1) || (RC1 == 1) || (RC3 == 1));

		// Turn off the LEDs
		CCP3CON = 0x00;		RA2 = 0;		// Turn PWM off and pin low
		CCP4CON = 0x00;		RC1 = 0;		// Turn PWM off and pin low
		CCP2CON = 0x00;		RC3 = 0;		// Turn PWM off and pin low
		RA5 = 1;							// Switch on the sensing circuit

		TMR0 = 0x80;						// It takes around 512uS for the sensing circuit to switch
		TMR0IF = 0;							// Different colors saturate the red LED by different amounts
		while(!TMR0IF);						// Making it trick to determine the minimal delay needed

		ADGO = 1;							// tell you what the red LED sees
		while(ADGO);
		RA5 = 0;							// Turn off the sensing circuit 
		new_photo = 1;
	
		// Enable the PWM system where we left off.  It seems that the 
		// CCP systems take a while to start up, so you need to pad some
		// time before the rollover of TMR2.  This is why TMR2 is started
		// at 0xF8 rather than the more obvious 0xFF.
		TMR2 = 0xF8;
		CCP3CON = 0x0C;						// Turn PWM back on
		CCP4CON = 0x0C;						// Turn PWM back on
		CCP2CON = 0x0C;						// Turn PWM back on
		// -----------------------Make a single ADC----------------------------

		if (i == PhotoArrayLength-1) new_photo_set = TRUE;
		photo_array[i] = ADRESH;
		i = (i+1) & 0x0F;

		TMR4 += 0xD8;						// 39 counts at 256uS/count = 10mS
		TMR4IF = 0;

	} // end timer 1 interrupt

} // end ISR

