/********************************************************/
/* Program         : iColor.h 							*/
/* Function        : Serial interface to Social Jewlery	*/
/* Author          : Chris Coulston     				*/
/* Platform        : HiTech (Lite Mode)  V9.71a			*/
/* Target          : PIC16LF1825 						*/
/*    			   : 14-pin SMT version					*/
/********************************************************/
#define BAUD 9600L      
#define FOSC 1000000L

#define DIVIDER ((int)(FOSC/(4UL * BAUD) -1))
#define HIGH_SPEED 1

#if HIGH_SPEED == 1
#define SPEED 0x4
#else
#define SPEED 0
#endif

#define RX_PIN TRISC5
#define TX_PIN TRISC4



/********************************************************/
/* 					Constants							*/
//		Delay	TMR0 cnts	Complement
// 		200uS	50			206		0xCE
//		500us	125			131		0x83
/********************************************************/
#define MAX_DUTY 		0x80
#define MAX_MASK 		0x7F
#define TenMilli		0xF63C
#define PhotoArrayLength	16
#define LogPhotoArrayLength	4
#define PhotoArrayMask		0x0F
//#define VarThreshold	16
//#define AvgChange		16
#define TRUE			1
#define FALSE			0
#define TwoHundredMicro	0xCE
#define OneMillisecond	0x06
#define SampleInterval	0xC0		
#define SampleLength	24
#define ZERO 			0
#define ONE				1

#define BAD_DATA		4
#define BAD_START		3
#define START			2
#define LOGIC1			1
#define LOGIC0			0



/********************************************************/
/* 				Global variables						*/
/********************************************************/
extern unsigned char array[SampleLength];
extern unsigned char pre;
extern unsigned char new_photo_set;
extern unsigned char photo_array[PhotoArrayLength];
extern unsigned char string[48], array[SampleLength];

/********************************************************/
/* 				Function Prototypes		main.c			*/
/********************************************************/
void INIT_PIC(void);
void interrupt MyISR(void);

/********************************************************/
/* 				Function Prototypes		util.c			*/
/********************************************************/
void Myprintf (char *ptr);
void putch(unsigned char byte);

/********************************************************/
/* 				Function Prototypes		color.c			*/
/********************************************************/
void TX(unsigned char send);
void JewleryTX(void);
void JewleryRX(void);


void Synch(void);
void SetTimer0Pre(void);
unsigned char DigitalRX(unsigned char process);
unsigned char DigitalClassifyBit(unsigned char low, unsigned char high);
unsigned char ALMOST(unsigned char actual, unsigned char time, unsigned char delta);
void DigitalRXArray(unsigned char *array);
unsigned char AnalogRX(void);
unsigned char AnalogReadBit(void);
unsigned char AnalogClassifyBit(unsigned char low, unsigned char high);
void RunSilent(void);
void RunActive(void);

void ColorCycle(void);
void ColorSat(unsigned char red, unsigned char green, unsigned char blue);
